import { AbstractNode } from "./aNode";
import { AbstractEdge } from "./aEdge";
import { Drawable } from "./drawable";
import { AbstractGrid } from "./aGrid";
/**
 * The class AbstractGraph is Abstract and extends the Drawable interface.
 * Logical graph representation:
 * - Keeps a reference to the nodes and edges on the graph.
 * - Draws the graph
 */
export declare abstract class AbstractGraph extends Drawable {
    protected nodes: AbstractNode[];
    protected edges: AbstractEdge[];
    protected otherDrawable: Drawable[];
    private static readonly GRAPH_PRIORITY;
    constructor();
    /**
     * Drawing the graph on a canvas referenced by @param ctx
     */
    draw(ctx: CanvasRenderingContext2D, grid: AbstractGrid): void;
    setupCanvas(canvas: HTMLCanvasElement): void;
    reset(): void;
    getEdges(): AbstractEdge[];
    getNodes(): AbstractNode[];
    update(ctx: CanvasRenderingContext2D, grid: AbstractGrid): void;
    /**
     * add @param node to the graph
     */
    addNode(node: AbstractNode): boolean;
    /**
     * delete @param node from the graph
     */
    deleteNode(node: AbstractNode): boolean;
    /**
     * add @param edge to the graph
     */
    addEdge(edge: AbstractEdge): boolean;
    /**
     * delete @param edge from the graph
     */
    deleteEdge(edge: AbstractEdge): boolean;
    abstract getEdge(startNode: AbstractNode, endNode: AbstractNode): AbstractEdge;
    /**
     * add @param drawable to be drawn on the graph
     */
    addDrawable(drawable: Drawable): boolean;
    /**
     * delete @param drawable to be drawn
     */
    deleteDrawable(drawable: Drawable): boolean;
}
