import { AbstractEdge } from "./abstract/aEdge";
import { AbstractGraph } from "./abstract/aGraph";
import { AbstractNode } from "./abstract/aNode";
/**
 * The GraphAlgorithm class is a collection of methods
 * that provides solutions to various graph problems.
 * Some of the problems encountered could be solved by implementing known
 * algorithms, such as Dijkstra's shortest path algorithm, Prim's minimum
 * spanning tree algorithm, etc. On the other hand,
 * some of the encountered problems were NP-complete,
 * such as the vertex cover problem as well as the Hamiltonian path
 * problem, which we solved by using brute-force algorithms.
 */
export declare class GraphAlgorithm {
    private static getNeighbors;
    /**
    * Upper bounds of finding paths. Finding > 5000 different paths lags heavely
    * On Server > 1000 different paths lags hard!
    */
    private static UPPERBOUNDPATHS;
    /**
     * Test the connectivity of the graph
     */
    static testConnectivity(graph: AbstractGraph): boolean;
    static testConnectivityViaEdges(edges: AbstractEdge[], graph: AbstractGraph): boolean;
    private static dfsForSelectedPathAndEdges;
    private static dfsForSelectedPathAndEdgesRec;
    static primMST(graph: AbstractGraph): AbstractEdge[];
    /**
    * @returns the minimum vertex cover
    * We compute every possible permutations of nodes and pick
    * the smallest possible set of nodes which fullfills the vertex cover
    */
    static vertexCover(graph: AbstractGraph): AbstractNode[];
    /**
     *
     * @returns a list of nodes set of size "size"
     */
    private static nodePermutation;
    private static permIndices;
    private static permIndicesRec;
    private static getMaxArray;
    static isVertexCover(cover: AbstractNode[], graph: AbstractGraph): boolean;
    static dijkstraShortestPath(graph: AbstractGraph, source: AbstractNode, target: AbstractNode): [AbstractNode[], AbstractEdge[]];
    private static smallestDistance;
    private static distanceUpdate;
    /**
   *
   * @param from, Node from which the path starts
   * @param to ,Node where the path ends
   * @returns true, if there exists a path from "from" to "to". false,else.
   */
    static depthFirstSearch(from: AbstractNode, to: AbstractNode, graph: AbstractGraph): boolean;
    /**
     *
     * @param from, Node from which the path starts
     * @param to, Node from where the path ends
     * @param graph, the graph containing both nodes
     * @returns all simple paths (no Node is beeing traversed twice) from "from" to "to". Nodes and edges from this path are beeing stored as a tuple
     */
    static allSimplePaths(source: AbstractNode, target: AbstractNode, graph: AbstractGraph, mandatoryNodes?: AbstractNode[]): [
        AbstractNode[],
        AbstractEdge[]
    ][];
    /**
     *
     * @param from, Node from which the path starts
     * @param to, Node from where the path ends
     * @param graph, the graph containing both nodes
     * @returns all simple paths (no Node is beeing traversed twice) from "from" to "to". Nodes and edges from this path are beeing stored as a tuple
     */
    static allSimplePathsFixedLength(source: AbstractNode, target: AbstractNode, graph: AbstractGraph, length: number, mandatoryNodes?: AbstractNode[], prefixNodes?: AbstractNode[]): [
        AbstractNode[],
        AbstractEdge[]
    ][];
    /**
     *
     * @param from, Node from which the path starts
     * @param to, Node from where the path ends
     * @param graph, the graph containing both nodes
     * @returns all paths from "from" to "to". Nodes and edges from this path are beeing stored as a tuple
     */
    static allPaths(source: AbstractNode, target: AbstractNode, graph: AbstractGraph, mandatoryNodes?: AbstractNode[]): [AbstractNode[], AbstractEdge[]][];
    static hamiltonPath(graph: AbstractGraph): [AbstractNode[], AbstractEdge[]];
    private static hamiltonPathRec;
    private static recDFS;
    private static cloneNodeArray;
    private static cloneEdgeArray;
    private static cloneNumberArray;
}
