import { Interactable } from "../abstract/interactable";
import { Grid } from "../base/grid";
import { Vertex } from "../base/vertex";
import { Edge } from "../base/edge";
import { Drawable } from "../abstract/drawable";
import { AbstractEdge } from "../abstract/aEdge";
import { AbstractNode } from "../abstract/aNode";
import { ActionHistory } from "./history";
import { InteractionType } from "./InteractionType";
import { AbstractGrid } from "../abstract/aGrid";
import { AbstractGraph } from "../abstract/aGraph";
export declare abstract class InteractiveGraph extends Interactable {
    protected readonly graph: AbstractGraph;
    protected readonly grid: Grid;
    protected ctx: CanvasRenderingContext2D;
    protected canvas: HTMLCanvasElement;
    private interactionType;
    private edgeVertex;
    protected selectColor: string;
    protected highLightColor: string;
    protected selectedNodes: AbstractNode[];
    protected selectedEdges: AbstractEdge[];
    protected sourceNode: AbstractNode;
    protected sourceNodeColor: string;
    protected targetNode: AbstractNode;
    protected targetNodeColor: string;
    protected mandatoryNodes: AbstractNode[];
    protected mandatoryNodeColor: string;
    protected prefixNodes: AbstractNode[];
    protected prefixNodeColor: string;
    protected weightingEdge: boolean;
    protected renamingNode: boolean;
    protected highLightNodes: AbstractNode[];
    protected highLightEdges: AbstractEdge[];
    protected history: ActionHistory;
    protected activDrawable: Interactable;
    protected callBackFunction: Function;
    constructor(graph: AbstractGraph, grid: Grid, canvas?: HTMLCanvasElement);
    setCanvas(canvas: HTMLCanvasElement): void;
    /**
    *
    * @param callbackFunction will be called every time an action on the graph is performed
    */
    setCallBackFunction(callbackFunction: (interactionType: InteractionType, object: any) => void): void;
    getNodes(): AbstractNode[];
    getEdges(): AbstractEdge[];
    getActive(): Interactable;
    getGrid(): AbstractGrid;
    getGraph(): AbstractGraph;
    forceUpdate(): void;
    reset(): void;
    resetGraph(): void;
    resetHighLightNodes(): void;
    /**
     *
     * @param nodes nodes to highlight, pass null to not highlight any nodes
     * @param edges edges to highlight, pass null to not highlight any edges
     */
    highLight(nodes: AbstractNode[], edges: AbstractEdge[], color?: string): void;
    highLightPath(nodes: AbstractNode[], edges?: AbstractEdge[], color?: string): void;
    undoAction(steps?: number): void;
    redoAction(): void;
    private mouseMoveHandler;
    private mouseOutHandler;
    private movable;
    private mouseOut;
    private mouseMove;
    private lastDrawable;
    private mouseMoveHover;
    leftClick(e: MouseEvent): void;
    /**
     *
     * @param coordinate coordinate on the canvas, where leftclick was performed
     */
    protected leftClickOnEmptySpace(coordinate: [number, number]): void;
    /**
     *
     * @param interacted Object on which a leftclick has been performed
     */
    protected leftClickOnObject(interacted: Interactable): void;
    private resetLocalStored;
    resetActive(): void;
    changeValue(active: Interactable, value: any): void;
    /**
     * Can be overwritten to disable hovering effects on certain objects
     * @param interactable, current object on which the mouse is pointed to
     */
    disableHover(interactable: Interactable): boolean;
    getValue(active: Interactable): any;
    abstract newNode(args?: any[]): AbstractNode;
    abstract newEdge(from: Vertex, to: Vertex, args?: [any]): Edge;
    private mouseEventHandler;
    private keyBoardEventHandler;
    add(drawable: Drawable, coordinate?: [number, number], round?: boolean, move?: boolean): void;
    private delete;
    private updateHandler;
    private doubleClickHandler;
    inPerimeter(coordinate: [number, number]): boolean;
    setInteractionType(type: InteractionType): void;
    getInteractionType(): InteractionType;
    setMandatoryNode(node: AbstractNode): void;
    setPrefixNode(node: AbstractNode): void;
    setSourceNode(node: AbstractNode): void;
    setSourceNodeColor(color: string): void;
    setTargetNode(node: AbstractNode): void;
    setTargetNodeColor(color: string): void;
    getSourceNode(): AbstractNode;
    getTargetNode(): AbstractNode;
    initSelected(startNode?: boolean): void;
    resetSelected(): void;
    getMandatoryNodes(): AbstractNode[];
    getPrefixNodes(): AbstractNode[];
    getSelectedNodes(): AbstractNode[];
    getSelectedEdges(): AbstractEdge[];
}
